<?php
class flowFlow extends flowModel
{
    /**
     * Construct for get not empty rule.
     * 
     * @param  string $appName 
     * @access public
     * @return void
     */
    public function __construct($appName = '')
    {
        parent::__construct($appName);
        $this->notEmptyRule = $this->loadModel('workflowrule')->getByTypeAndRule('system', 'notempty');
    }

    /**
     * Get menu of a flow.
     *
     * @param  object $flow
     * @param  array  $labels
     * @access public
     * @return void
     */
    public function getModuleMenu($flow, $labels)
    {
        if($flow->navigator == 'primary' && isset($this->lang->{$flow->module}->menu)) $this->lang->flow->menu = $this->lang->{$flow->module}->menu;
        if($flow->navigator == 'secondary' && isset($this->lang->{$flow->app}->menu))  $this->lang->flow->menu = $this->lang->{$flow->app}->menu;
        return false;
    }

    /**
     * Post data of a flow.
     *
     * @param  object $flow
     * @param  object $action
     * @param  int    $id
     * @access public
     * @return array
     */
    public function post($flow, $action, $id = 0)
    {
        $result = parent::post($flow, $action, $id);

        if(isset($result['result']) and $result['result'] == 'success' && $action->open == 'modal')
		{
			$result['locate'] = 'parent';
		}
		elseif(isset($result['result']) and $result['result'] == 'success' && $flow->buildin == 1 and $id > 0)
		{
			$result['locate'] = helper::createLink($flow->module, 'view', "id={$id}");
		}

        return $result;
    }

    /**
     * Print workflow defined fields for view and form page.
     * 
     * @param  string $moduleName 
     * @param  string $methodName 
     * @param  object $object       bug | build | feedback | product | productplan | project | release | story | task | testcase | testsuite | testtask
     * @param  string $type         The parent component which fileds displayed in. It should be table or div.
     * @param  string $extras       The extra params.
     *                              columns=1|2|3|5         Number of the columns merged to display the fields. The default is 1.
     *                              position=left|right     The position which the fields displayed in a page.
     *                              inForm=0|1              The fields displayed in a form or not. The default is 1.
     *                              inCell=0|1              The fields displayed in a div with class cell or not. The default is 0.
     * @access public
     * @return string
     */
    public function printFields($moduleName, $methodName, $object, $type, $extras = '')
    {
        parse_str(str_replace(array(',', ' '), array('&', ''), $extras), $params);

        $function = "printFieldsIn" . $type;
        $fields   = $this->loadModel('workflowaction')->getFields($moduleName, $methodName);
        $layouts  = $this->loadModel('workflowlayout')->getFields($moduleName, $methodName);

        return $this->$function($object, $layouts, $fields, $params);
    }

    /**
     * Print fields in table.
     * 
     * @param  object   $object 
     * @param  array    $layouts 
     * @param  array    $fields
     * @param  array    $params
     * @access public
     * @return string
     */
    public function printFieldsInTable($object, $layouts, $fields, $params = '')
    {
        $html    = '';
        $columns = zget($params, 'columns', 1);
        $inForm  = zget($params, 'inForm', 1);
        $colspan = $columns > 1 ? "colspan='$columns'" : '';

        if(!is_object($object)) $object = (object)$object;
        if(!$object) $object = new stdclass();

        foreach($fields as $field)
        {
            if($field->buildin or !$field->show or !isset($layouts[$field->field])) continue;

            if(!isset($object->{$field->field})) $object->{$field->field} = '';

            $require = '';
            if($inForm && $this->notEmptyRule && strpos(",$field->rules,", ",{$this->notEmptyRule->id},") !== false) $require = "class='required'";

            $content = $inForm ? $this->getFieldControl($field, $object) : $this->getFieldValue($field, $object);
            $html .= "<tr><th>{$field->name}</th>"; 
            $html .= "<td $colspan $require>{$content}</td></tr>"; 
        }

        return $html;
    }

    /**
     * Print fields in div.
     * 
     * @param  object $object 
     * @param  array  $layouts 
     * @param  array  $fields 
     * @param  array  $params 
     * @access public
     * @return string
     */
    public function printFieldsInDiv($object, $layouts, $fields, $params = '')
    {
        $html     = '';
        $position = zget($params, 'position', 'right');
        $inCell   = zget($params, 'inCell', 0);
        $inForm   = zget($params, 'inForm', 1);

        if($position == 'right')
        {
            if($inCell) $html .= "<div class='cell'>"; 

            $html .= "<div class='detail'>"; 
            $html .= "<div class='detail-title'>{$this->lang->extInfo}</div>"; 
            $html .= $inCell ? "<table class='table table-data'>" : "<table class='table table-form'>"; 

            $tableContent = '';
            foreach($fields as $field)
            {
                if($field->buildin or !$field->show or !isset($layouts[$field->field]) or $field->position != 'basic') continue;

                $require = '';
                if($inForm && $this->notEmptyRule && strpos(",$field->rules,", ",{$this->notEmptyRule->id},") !== false) $require = "class='required'";

                $content = $inForm ? $this->getFieldControl($field, $object) : $this->getFieldValue($field, $object);

                $tableContent .= "<tr><th class='thWidth'>{$field->name}</th>"; 
                $tableContent .= "<td $require>{$content}</td></tr>"; 
            }

            if(!$tableContent) return false;

            $html .= $tableContent;
            $html .= '</table>'; 
            $html .= '</div>'; 

            if($inCell) $html .= '</div>'; 
        }

        if($position == 'left')
        {
            foreach($fields as $field)
            {
                if($field->buildin or !$field->show or !isset($layouts[$field->field]) or $field->position != 'info') continue;

                $require = '';
                if($inForm && $this->notEmptyRule && strpos(",$field->rules,", ",{$this->notEmptyRule->id},") !== false) $require = "required";

                $content = $inForm ? $this->getFieldControl($field, $object) : $this->getFieldValue($field, $object);

                if($inCell) $html .= "<div class='cell'>"; 

                $html .= "<div class='detail'>"; 
                $html .= "<div class='detail-title'>{$field->name}</div>"; 
                $html .= "<div class='detail-content $require'>{$content}</div>"; 
                $html .= '</div>'; 

                if($inCell) $html .= '</div>'; 
            }
        }

        return $html;
    }

    /**
     * Get control of a field.
     * 
     * @param  object    $field 
     * @param  object    $object 
     * @access public
     * @return string
     */
    public function getFieldControl($field, $object)
    {
        $control = '';

        if($field->control == 'select')   $control = html::select($field->field, $field->options, $object->{$field->field}, "class='form-control chosen'"); 
        if($field->control == 'checkbox' or $field->control == 'radio' and isset($field->options[''])) unset($field->options['']);
        if($field->control == 'checkbox') $control = html::checkbox($field->field, $field->options, $object->{$field->field}, "class='form-control'", 'inline'); 
        if($field->control == 'radio')    $control = html::radio($field->field, $field->options, $object->{$field->field}); 
        if($field->control == 'textarea') $control = html::textarea($field->field, $object->{$field->field}, 'class="form-control"'); 
        if($field->control == 'input')    $control = html::input($field->field, $object->{$field->field}, 'class="form-control"'); 
        if($field->control == 'date')     $control = html::input($field->field, $object->{$field->field}, 'class="form-control form-date"'); 
        if($field->control == 'datetime') $control = html::input($field->field, $object->{$field->field}, 'class="form-control form-datetime"'); 

        if($field->field == 'subStatus')  $control .= $this->getSubStatusScript($field);
        
        return $control;
    }

    /**
     * Get value string of one field.
     * 
     * @param  object    $field 
     * @param  object    $object 
     * @access public
     * @return string
     */
    public function getFieldValue($field, $object)
    {
        if($field->control != 'checkbox') return zget($field->options, $object->{$field->field}, $object->{$field->field});

        $content = '';
        $values  = explode(',', str_replace(' ', '', $object->{$field->field}));
        foreach($values as $value) $content .= ' ' . zget($field->options, $value, $value);

        return $content;
    }

    /**
     * Print workflow defined fields from browse page.
     * 
     * @param string $module
     * @param object $object
     * @param string $id
     * @access public
     * @return void
     */
    public function printFlowCell($module, $object, $id)
    {
        $field = $this->loadModel('workflowfield')->getByField($module, $id);
        if(isset($field->buildin) && $field->buildin == 0)
        {
            if(strpos('select,radio,checkbox', $field->control) === false) 
            {
                echo $object->{$field->field};
            }
            else
            {
                $options = $this->workflowfield->getFieldOptions($field);
                echo zget($options, $object->{$field->field});
            }
        }
    }
}
