<?php
/**
 * The model file of workflowaction module of RanZhi.
 *
 * @copyright   Copyright 2009-2016 青岛易软天创网络科技有限公司(QingDao Nature Easy Soft Network Technology Co,LTD, www.cnezsoft.com)
 * @license     商业软件，非开源软件
 * @author      Gang Liu <liugang@cnezsoft.com>
 * @package     workflowaction
 * @version     $Id$
 * @link        http://www.ranzhico.com
 */
class workflowactionModel extends model
{
    /**
     * Get an action by id.
     *
     * @param  int    $id
     * @access public
     * @return object
     */
    public function getByID($id)
    {
        $action = $this->dao->select('*')->from(TABLE_WORKFLOWACTION)->where('id')->eq($id)->fetch();
        if($action)
        {
            $action->conditions    = json_decode($action->conditions);
            $action->verifications = json_decode($action->verifications);
            /* Make sure hooks is an indexed array. */
            $action->hooks         = array_values((array)json_decode($action->hooks));
        }

        return $action;
    }

    /**
     * Get an action by module and action.
     *
     * @param  string $module
     * @param  string $action
     * @access public
     * @return object
     */
    public function getByModuleAndAction($module, $action)
    {
        $action = $this->dao->select('*')->from(TABLE_WORKFLOWACTION)->where('module')->eq($module)->andWhere('action')->eq($action)->fetch();

        if($action)
        {
            if(is_string($action->conditions))    $action->conditions    = json_decode($action->conditions);
            if(is_string($action->verifications)) $action->verifications = json_decode($action->verifications);
            if(is_string($action->hooks))         $action->hooks         = json_decode($action->hooks);
            /* Make sure hooks is an indexed array. */
            $action->hooks = array_values((array)$action->hooks);
        }

        return $action;
    }

    /**
     * Get action list.
     *
     * @param  string $module
     * @param  string $orderBy
     * @access public
     * @return array
     */
    public function getList($module, $orderBy = 'id')
    {
        $actions = $this->dao->select('*')->from(TABLE_WORKFLOWACTION)->where('module')->eq($module)->orderBy($orderBy)->fetchAll('id');

        foreach($actions as $action)
        {
            if(is_string($action->conditions))    $action->conditions    = json_decode($action->conditions);
            if(is_string($action->verifications)) $action->verifications = json_decode($action->verifications);
            if(is_string($action->hooks))         $action->hooks         = json_decode($action->hooks);
            /* Make sure hooks is an indexed array. */
            $action->hooks = array_values((array)$action->hooks);
        }

        return $actions;
    }

    /**
     * Get action list by group.
     *
     * @param  string $orderBy
     * @access public
     * @return array
     */
    public function getGroupList($orderBy = 'id')
    {
        return $this->dao->select('*')->from(TABLE_WORKFLOWACTION)->orderBy($orderBy)->fetchGroup('module');
    }

    /**
     * Get action pairs.
     *
     * @param  string $module
     * @access public
     * @return array
     */
    public function getPairs($module)
    {
        $actions = $this->dao->select('id, name')->from(TABLE_WORKFLOWACTION)->where('module')->eq($module)->fetchPairs();

        return array('' => '') + $actions;
    }

    /**
     * Get fields of an action.
     *
     * @param  string $module
     * @param  string $action
     * @param  bool   $getRealOptions
     * @access public
     * @return array
     */
    public function getFields($module, $action, $getRealOptions = true)
    {
        $actionFields = $this->dao->select("t1.*, t2.name, t2.control, t2.options, t2.default, t2.type, CONCAT(t1.layoutRules, ',', t2.rules) AS rules, t2.placeholder, t2.canSearch, t2.desc, t2.buildin")
            ->from(TABLE_WORKFLOWLAYOUT)->alias('t1')
            ->leftJoin(TABLE_WORKFLOWFIELD)->alias('t2')->on('t1.module=t2.module AND t1.field=t2.field')
            ->where('t1.module')->eq($module)
            ->andWhere('t1.action')->eq($action)
            ->orderBy('t1.order')
            ->fetchAll('field');

        $action    = $this->getByModuleAndAction($module, $action);
        $flow      = $this->loadModel('workflow', 'flow')->getByModule($module);
        $subTables = $this->workflow->getPairs($module, $type = 'table');
        $fieldList = $this->loadModel('workflowfield', 'flow')->getList($module);

        foreach($actionFields as $field) $field = $this->workflowfield->processFieldOptions($field);

        $fields = array();
        if(empty($actionFields))
        {
            foreach($fieldList as $field)
            {
                if(!$field) continue;

                $options = $field->options;
                if($getRealOptions) $options = $this->getRealOptions($field);

                $fields[$field->field] = new stdclass();
                $fields[$field->field]->module       = $module;
                $fields[$field->field]->field        = $field->field;
                $fields[$field->field]->type         = $field->type;
                $fields[$field->field]->name         = $field->name;
                $fields[$field->field]->control      = $field->control;
                $fields[$field->field]->buildin      = $field->buildin;
                $fields[$field->field]->show         = '0';
                $fields[$field->field]->width        = 'auto';
                $fields[$field->field]->position     = '';
                $fields[$field->field]->readonly     = '0';
                $fields[$field->field]->mobileShow   = '0';
                $fields[$field->field]->totalShow    = '0';
                $fields[$field->field]->defaultValue = '';
                $fields[$field->field]->rules        = '';
                $fields[$field->field]->layoutRules  = '';
                $fields[$field->field]->options      = $options;
            }

            if($action and $action->action == 'browse')
            {
                $fields['actions'] = new stdclass();
                $fields['actions']->module       = $module;
                $fields['actions']->field        = 'actions';
                $fields['actions']->type         = '';
                $fields['actions']->name         = $this->lang->actions;
                $fields['actions']->control      = '';
                $fields['actions']->buildin      = 1;
                $fields['actions']->show         = '1';
                $fields['actions']->width        = '160';
                $fields['actions']->position     = '';
                $fields['actions']->readonly     = '0';
                $fields['actions']->mobileShow   = '0';
                $fields['actions']->totalShow    = '0';
                $fields['actions']->defaultValue = '';
                $fields['actions']->rules        = '';
                $fields['actions']->layoutRules  = '';
                $fields['actions']->options      = '';
            }

            if($flow && $flow->type == 'flow' && $action && $action->action != 'browse' && $action->type != 'batch')
            {
                $fields['file'] = new stdclass();
                $fields['file']->module       = $module;
                $fields['file']->field        = 'file';
                $fields['file']->type         = '';
                $fields['file']->name         = $this->lang->files;
                $fields['file']->control      = '';
                $fields['file']->buildin      = 1;
                $fields['file']->show         = '0';
                $fields['file']->width        = 'auto';
                $fields['file']->position     = '';
                $fields['file']->readonly     = '0';
                $fields['file']->mobileShow   = '0';
                $fields['file']->totalShow    = '0';
                $fields['file']->defaultValue = '';
                $fields['file']->rules        = '';
                $fields['file']->layoutRules  = '';
                $fields['file']->options      = '';

                foreach($subTables as $subModule => $subName)
                {
                    /* 为了把明细表代号和主表字段区分开，增加前缀。In order to distinguish the schedule code from the main table field, add a prefix. */
                    $subModule = 'sub_' . $subModule;

                    $fields[$subModule] = new stdclass();
                    $fields[$subModule]->module       = $module;
                    $fields[$subModule]->field        = $subModule;
                    $fields[$subModule]->type         = '';
                    $fields[$subModule]->name         = $subName;
                    $fields[$subModule]->control      = '';
                    $fields[$subModule]->buildin      = 1;
                    $fields[$subModule]->show         = '0';
                    $fields[$subModule]->width        = 'auto';
                    $fields[$subModule]->position     = '';
                    $fields[$subModule]->readonly     = '0';
                    $fields[$subModule]->mobileShow   = '0';
                    $fields[$subModule]->totalShow    = '0';
                    $fields[$subModule]->defaultValue = '';
                    $fields[$subModule]->rules        = '';
                    $fields[$subModule]->layoutRules  = '';
                    $fields[$subModule]->options      = array();
                }
            }
        }
        else
        {
            foreach($actionFields as $key => $field)
            {
                if(!$field) continue;

                $options = $field->options;
                if($getRealOptions) $options = $this->getRealOptions($field);

                $fields[$key] = new stdclass();
                $fields[$key]->module       = $module;
                $fields[$key]->field        = $field->field;
                $fields[$key]->type         = $field->type;
                $fields[$key]->name         = zget($subTables, str_replace('sub_', '', $key), $field->name);    // 把前缀去掉来获得明细表名。
                $fields[$key]->control      = $field->control;
                $fields[$key]->buildin      = $field->buildin;
                $fields[$key]->show         = $action->action == 'browse' && !$field->canSearch ? '0' : '1';
                $fields[$key]->width        = $field->width ? $field->width : 'auto';
                $fields[$key]->position     = $field->position;
                $fields[$key]->readonly     = $field->readonly;
                $fields[$key]->mobileShow   = $field->mobileShow;
                $fields[$key]->totalShow    = $field->totalShow;
                $fields[$key]->defaultValue = empty($field->defaultValue) ? $field->default : $field->defaultValue;
                $fields[$key]->rules        = $field->rules;
                $fields[$key]->layoutRules  = $field->layoutRules;
                $fields[$key]->options      = $options;

                if($key == 'file')    $fields[$key]->name = $this->lang->files;
                if($key == 'actions') $fields[$key]->name = $this->lang->actions;
                if($key == 'actions' && $action->action == 'browse') $fields[$key]->show = 1;
            }

            foreach($fieldList as $id => $field)
            {
                if(!$field or isset($actionFields[$field->field])) continue;

                $options = $field->options;
                if($getRealOptions) $options = $this->getRealOptions($field);

                $fields[$field->field] = new stdclass();
                $fields[$field->field]->module       = $module;
                $fields[$field->field]->field        = $field->field;
                $fields[$field->field]->type         = $field->type;
                $fields[$field->field]->name         = $field->name;
                $fields[$field->field]->control      = $field->control;
                $fields[$field->field]->buildin      = $field->buildin;
                $fields[$field->field]->show         = '0';
                $fields[$field->field]->width        = 'auto';
                $fields[$field->field]->position     = '';
                $fields[$field->field]->readonly     = '0';
                $fields[$field->field]->mobileShow   = '0';
                $fields[$field->field]->totalShow    = '0';
                $fields[$field->field]->defaultValue = '';
                $fields[$field->field]->rules        = '';
                $fields[$field->field]->layoutRules  = '';
                $fields[$field->field]->options      = $options;
            }

            if($flow->type == 'flow' && $action->action != 'browse' && $action->type != 'batch')
            {
                if(!isset($fields['file']))
                {
                    $fields['file'] = new stdclass();
                    $fields['file']->module       = $module;
                    $fields['file']->field        = 'file';
                    $fields['file']->type         = '';
                    $fields['file']->name         = $this->lang->files;
                    $fields['file']->control      = '';
                    $fields['file']->buildin      = 1;
                    $fields['file']->show         = '0';
                    $fields['file']->width        = 'auto';
                    $fields['file']->position     = '';
                    $fields['file']->readonly     = '0';
                    $fields['file']->mobileShow   = '0';
                    $fields['file']->totalShow    = '0';
                    $fields['file']->defaultValue = '';
                    $fields['file']->rules        = '';
                    $fields['file']->layoutRules  = '';
                    $fields['file']->options      = '';
                }

                foreach($subTables as $subModule => $subName)
                {
                    if(!$subModule) continue;

                    /* 为了把明细表代号和主表字段区分开，增加前缀。In order to distinguish the schedule code from the main table field, add a prefix. */
                    if($subModule) $subModule = 'sub_' . $subModule;
                    if(isset($actionFields[$subModule])) continue;

                    $fields[$subModule] = new stdclass();
                    $fields[$subModule]->module       = $module;
                    $fields[$subModule]->field        = $subModule;
                    $fields[$subModule]->type         = '';
                    $fields[$subModule]->name         = $subName;
                    $fields[$subModule]->control      = '';
                    $fields[$subModule]->show         = '0';
                    $fields[$subModule]->width        = 'auto';
                    $fields[$subModule]->position     = '';
                    $fields[$subModule]->readonly     = '0';
                    $fields[$subModule]->mobileShow   = '0';
                    $fields[$subModule]->totalShow    = '0';
                    $fields[$subModule]->defaultValue = '';
                    $fields[$subModule]->rules        = '';
                    $fields[$subModule]->layoutRules  = '';
                    $fields[$subModule]->options      = array();
                }
            }
        }

        return $fields;
    }

    /**
     * Get real options.
     *
     * @param  object $field
     * @access public
     * @return array
     */
    public function getRealOptions($field)
    {
        $options = $this->loadModel('workflowfield', 'flow')->getFieldOptions($field);

        $this->app->loadLang('workflowlayout', 'flow');
        if($field->options == 'user')
        {
            $options = $this->lang->workflowlayout->default->user + $options;
        }
        elseif($field->options == 'dept')
        {
            $options = $this->lang->workflowlayout->default->dept + $options;
        }
        elseif($field->control == 'date' || $field->control == 'datetime')
        {
            $options = $this->lang->workflowlayout->default->time + $options;
        }

        return $options;
    }

    /**
     * Get users to notice.
     *
     * @param  string $module
     * @access public
     * @return array
     */
    public function getUsers2Notice($module)
    {
        $fields = $this->loadModel('workflowfield', 'flow')->getList($module);

        $this->app->loadLang('workflowdatasource', 'flow');
        $toList = array('deptManager' => $this->lang->workflowdatasource->default->options['deptManager']);
        if($fields)
        {
            foreach($fields as $field)
            {
                if($field->options == 'user') $toList[$field->field] = $field->name;
            }
        }

        return $toList + $this->loadModel('user')->getDeptPairs('nodeleted,noforbidden,noclosed');
    }

    /**
     * Create an action.
     *
     * @access public
     * @return bool | int
     */
    public function create()
    {
        $action = fixer::input('post')
            ->add('extensionType', 'override')
            ->add('conditions', '[]')
            ->add('hooks', '[]')
            ->add('createdBy', $this->app->user->account)
            ->add('createdDate', helper::now())
            ->setIF($this->post->type == 'batch', 'position', 'browse')
            ->setIF($this->post->type == 'batch', 'show', 'direct')
            ->setForce('action', strtolower(str_replace(' ', '', $this->post->action)))
            ->get();

        if(!empty($action->action) && !validater::checkREG($action->action, '|^[a-z]+$|'))
        {
            dao::$errors['action'][] = sprintf($this->lang->workflowaction->error->wrongCode, $this->lang->workflowaction->action);
        }

        if(dao::isError()) return false;

        $this->dao->insert(TABLE_WORKFLOWACTION)->data($action)
            ->autoCheck()
            ->batchCheck($this->config->workflowaction->require->create, 'notempty')
            ->batchCheck($this->config->workflowaction->uniqueFields, 'unique', "module='$action->module'")
            ->exec();

        if(dao::isError()) return false;

        $actionID = $this->dao->lastInsertId();

        /* Create the actionBy and actionDate fields. */
        if($action->action)
        {
            $action->id = $actionID;
            $result = $this->createFields($action);
            if(!$result) $this->delete($actionID);
        }

        return $actionID;
    }

    /**
     * Create the actionBy and actionDate fields for an action.
     *
     * @param  object $action
     * @access public
     * @return bool
     */
    public function createFields($action)
    {
        $code  = $action->action . 'By';
        $flow  = $this->loadModel('workflow')->getByModule($action->module);
        $field = $this->loadModel('workflowfield')->getByField($flow->module, $code);
        if(!$field)
        {
            $sql = "ALTER TABLE `$flow->table` ADD `$code` varchar(30) NOT NULL";

            try
            {
                $this->dbh->query($sql);

                $orderField = $this->workflowfield->getLastField($flow->module);
                $order      = $orderField->order;

                $field = new stdclass();
                $field->module      = $flow->module;
                $field->field       = $code;
                $field->type        = 'varchar';
                $field->length      = 30;
                $field->name        = sprintf($this->lang->workflowaction->actionBy, $action->name);
                $field->control     = 'select';
                $field->options     = 'user';
                $field->order       = $order + 1;
                $field->createdBy   = $this->app->user->account;
                $field->createdDate = helper::now();
                $field->readonly    = '1';  // Make sure user can not change the field.

                $this->dao->insert(TABLE_WORKFLOWFIELD)->data($field)->exec();

                $this->dao->update(TABLE_WORKFLOWFIELD)->set('`order` = `order` + 1')
                    ->where('module')->eq($flow->module)
                    ->andWhere('field')->ne($code)
                    ->andWhere('`order`')->gt($order)
                    ->exec();

                if(dao::isError())
                {
                    $sql = "ALTER TABLE `$flow->table` DROP `$code`";
                    $this->dbh->query($sql);
                }
            }
            catch(PDOException $exception)
            {
                dao::$errors[] = $exception->getMessage();
            }
        }

        if(dao::isError()) return false;

        $code  = $action->action . 'Date';
        $field = $this->workflowfield->getByField($flow->module, $code);
        if(!$field)
        {
            $sql = "ALTER TABLE `$flow->table` ADD `$code` datetime NOT NULL";
            try
            {
                $this->dbh->query($sql);

                $orderField = $this->workflowfield->getLastField($flow->module);
                $order      = $orderField->order;

                $field = new stdclass();
                $field->module      = $flow->module;
                $field->field       = $code;
                $field->type        = 'datetime';
                $field->name        = sprintf($this->lang->workflowaction->actionDate, $action->name);
                $field->control     = 'datetime';
                $field->options     = '[]';
                $field->order       = $order + 1;
                $field->createdBy   = $this->app->user->account;
                $field->createdDate = helper::now();
                $field->readonly    = '1';

                $this->dao->insert(TABLE_WORKFLOWFIELD)->data($field)->exec();

                $this->dao->update(TABLE_WORKFLOWFIELD)->set('`order` = `order` + 1')
                    ->where('module')->eq($flow->module)
                    ->andWhere('field')->ne($code)
                    ->andWhere('`order`')->gt($order)
                    ->exec();

                if(dao::isError())
                {
                    $sql = "ALTER TABLE `$flow->table` DROP `$code`";
                    $this->dbh->query($sql);
                }
            }
            catch(PDOException $exception)
            {
                dao::$errors[] = $exception->getMessage();
            }
        }

        return !dao::isError();
    }

    /**
     * Update an action.
     *
     * @param  int    $id
     * @access public
     * @return array
     */
    public function update($id)
    {
        $oldAction = $this->getByID($id);

        $action = fixer::input('post')
            ->add('editedBy', $this->app->user->account)
            ->add('editedDate', helper::now())
            ->setIF($this->post->type == 'batch', 'position', 'browse')
            ->setIF($this->post->type == 'batch', 'show', 'direct')
            ->setIF($this->post->action, 'action', strtolower(str_replace(' ', '', $this->post->action)))
            ->get();

        if(!empty($action->action) && !validater::checkREG($action->action, '|^[A-Za-z]+$|'))
        {
            dao::$errors['action'][] = sprintf($this->lang->workflowaction->error->wrongCode, $this->lang->workflowaction->action);
        }

        if(dao::isError()) return;

        $isDefault = in_array($oldAction->action, $this->config->workflowaction->defaultActions);

        $this->dao->update(TABLE_WORKFLOWACTION)->data($action)
            ->where('id')->eq($id)
            ->autoCheck()
            ->batchCheckIF($isDefault, 'name', 'notempty')
            ->batchCheckIF(!$isDefault, $this->config->workflowaction->require->edit, 'notempty')
            ->batchCheck($this->config->workflowaction->uniqueFields, 'unique', "id!='$id' AND module='$action->module'")
            ->exec();

        return commonModel::createChanges($oldAction, $action);
    }

    /**
     * Delete an action.
     *
     * @param  int    $id
     * @param  object $null
     * @access public
     * @return bool
     */
    public function delete($id, $null = null)
    {
        $action = $this->getByID($id);
        if(!$action) return true;

        $this->dao->delete()->from(TABLE_WORKFLOWLAYOUT)->where('module')->eq($action->module)->andWhere('action')->eq($action->action)->exec();
        $this->dao->delete()->from(TABLE_WORKFLOWACTION)->where('id')->eq($id)->exec();

        return !dao::isError();
    }

    /**
     * Save condition of an action.
     *
     * @param  int    $id
     * @access public
     * @return void
     */
    public function saveCondition($id)
    {
        $condition = new stdclass();
        $condition->conditionType = $this->post->conditionType;
        $condition->fields        = array();
        $condition->sql           = $this->post->sql;
        $condition->sqlResult     = $this->post->sqlResult;

        if($condition->conditionType == 'sql')
        {
            if(!$condition->sql)
            {
                dao::$errors['sql'] = sprintf($this->lang->error->notempty, $this->lang->workflowcondition->sql);
            }
            else
            {
                $checkResult = $this->loadModel('workflowfield', 'flow')->checkSqlAndVars($condition->sql);
                if($checkResult !== true) dao::$errors['sql'] = $checkResult;
            }
        }
        else
        {
            foreach($this->post->field as $key => $data)
            {
                if(empty($data)) continue;

                $field = new stdclass();
                $field->field           = $data;
                $field->operator        = $this->post->operator[$key];
                $field->param           = $this->post->param[$key];
                $field->logicalOperator = $this->post->logicalOperator[$key];

                $condition->fields[] = $field;
            }
        }

        $condition = helper::jsonEncode($condition);
        $this->dao->update(TABLE_WORKFLOWACTION)->set('conditions')->eq($condition)->where('id')->eq($id)->exec();

        return !dao::isError();
    }

    /**
     * Save verification of an action.
     *
     * @param  int    $id
     * @access public
     * @return bool
     */
    public function saveVerification($id)
    {
        if(!$this->post->message) return array('result' => 'fail', 'message' => array('message' => sprintf($this->lang->error->notempty, $this->lang->workflowverification->message)));

        $errors     = array();
        $sqlVars    = array();
        $formVars   = array();
        $recordVars = array();

        $data = new stdclass();
        $data->type    = $this->post->type;
        $data->message = $this->post->message;

        if($data->type == 'data')
        {
            $fields = array();
            foreach($this->post->verifications['field'] as $key => $field)
            {
                if(!$field) continue;
                $verification = new stdclass();
                $verification->field           = $field;
                $verification->logicalOperator = $this->post->verifications['logicalOperator'][$key];
                $verification->operator        = $this->post->verifications['operator'][$key];
                $verification->paramType       = $this->post->verifications['paramType'][$key];
                if($verification->paramType == 'form')
                {
                    $verification->param = $this->post->verifications['param'][$key];
                    $formVars[$field] = $this->post->verifications['param'][$key];
                }
                elseif($verification->paramType == 'record')
                {
                    $verification->param = $this->post->verifications['param'][$key];
                    $recordVars[$field]  = $this->post->verifications['param'][$key];
                }
                elseif(!empty($verification->paramType) && strpos(',today,now,actor,deptManager,', ",$verification->paramType,") !== false)
                {
                    $verification->param = $verification->paramType;
                    $sqlVars[$field]     = $verification->paramType;
                }
                else
                {
                    $verification->param = $this->post->verifications['param'][$key];
                }
                $fields[] = $verification;
            }

            $data->fields     = $fields;
            $data->sqlVars    = $sqlVars;
            $data->formVars   = $formVars;
            $data->recordVars = $recordVars;
        }
        elseif($data->type == 'sql')
        {
            if(!$this->post->sql)
            {
                $errors['sql'] = sprintf($this->lang->error->notempty, $this->lang->workflowverification->sql);
            }
            else
            {
                $vars      = array();
                $varValues = array();
                foreach($this->post->varName as $key => $varName)
                {
                    if(!$varName) continue;

                    $var = new stdclass();
                    $var->varName   = $varName;
                    $var->paramType = $this->post->paramType[$key];
                    $var->param     = $this->post->param[$key];

                    $varValues[$varName] = $var->param;
                    $vars[] = $var;
                }

                $checkResult = $this->loadModel('workflowfield', 'flow')->checkSqlAndVars($this->post->sql, $varValues);
                if($checkResult !== true) $errors['sql'] = $checkResult;

                $data->sql       = $this->post->sql;
                $data->sqlVars   = $vars;
                $data->sqlResult = $this->post->sqlResult;
            }
        }

        if(!empty($errors)) return array('result' => 'fail', 'message' => $errors);

        $this->dao->update(TABLE_WORKFLOWACTION)
            ->set('verifications')->eq(helper::jsonEncode($data))
            ->autoCheck()
            ->where('id')->eq($id)
            ->exec();
        return !dao::isError();
    }

    /**
     * Save notice of an action.
     *
     * @param  int    $id
     * @access public
     * @return bool
     */
    public function saveNotice($id)
    {
        $toList = ',' . trim(implode(',', $this->post->toList), ',') . ',';
        $this->dao->update(TABLE_WORKFLOWACTION)->set('toList')->eq($toList)->where('id')->eq($id)->exec();
        return !dao::isError();
    }

    /**
     * Check if the button is clickable.
     *
     * @param  object $action
     * @param  string $methodName
     * @access public
     * @return bool
     */
    public function isClickable($action, $methodName)
    {
        $actionConfig = $this->config->workflowaction;
        $methodName   = strtolower($methodName);

        if($methodName == 'setcondition')    return (($action->buildin && $action->extensionType == 'override') or (!$action->buildin && !in_array($action->action, $actionConfig->noConditionActions)));
        if($methodName == 'setverification') return ($action->open != 'none' && (($action->buildin && $action->extensionType == 'override') or (!$action->buildin && !in_array($action->action, $actionConfig->noVerificationActions))));
        if($methodName == 'setnotice')       return (($action->buildin && $action->extensionType == 'override') or (!$action->buildin && !in_array($action->action, $actionConfig->noNoticeActions)));
        /* workflowlayout */
        if($methodName == 'admin')           return ($action->open != 'none' && (($action->buildin && $action->extensionType == 'override') or (!$action->buildin && $action->action != 'export') or ($action->buildin && $action->extensionType == 'extend')));
        /* workflowhook */
        if($methodName == 'browse')          return (($action->buildin && $action->extensionType != 'none') or (!$action->buildin && !in_array($action->action, $actionConfig->noHookActions)));
    }
}
