<?php
/**
 * The model file of workflowfield module of RanZhi.
 *
 * @copyright   Copyright 2009-2016 青岛易软天创网络科技有限公司(QingDao Nature Easy Soft Network Technology Co,LTD, www.cnezsoft.com)
 * @license     商业软件，非开源软件
 * @author      Gang Liu <liugang@cnezsoft.com>
 * @package     workflowfield
 * @version     $Id$
 * @link        http://www.ranzhico.com
 */
class workflowfieldModel extends model
{
    /**
     * Check sql vars.
     *
     * @param  string $sql
     * @param  string $sign
     * @access public
     * @return array
     */
    public function checkSqlVar($sql = '', $sign = '\$')
    {
        $sql = $sql . ' ';
        preg_match_all("/{$sign}(\w+)/i", $sql, $out);
        return array_unique($out[1]);
    }

    /**
     * replace defined table names.
     *
     * @param  string $sql
     * @access public
     * @return void
     */
    public function replaceTableNames($sql)
    {
        if(preg_match_all("/TABLE_[A-Z]+/", $sql, $out))
        {
            rsort($out[0]);
            foreach($out[0] as $table)
            {
                if(!defined($table)) continue;
                $sql = str_replace($table, trim(constant($table), '`'), $sql);
            }
        }
        $sql = preg_replace("/= *'\!/U", "!='", $sql);
        return $sql;
    }

    /**
     * Check input sql and vars.
     *
     * @param  string $sql
     * @param  array  $vars
     * @access public
     * @return string || bool
     */
    public function checkSqlAndVars($sql = '', $vars = array())
    {
        $sqlVars = $this->checkSqlVar($sql, '\$');
        if($sqlVars)
        {
            foreach($sqlVars as $sqlVar)
            {
                if(isset($vars[$sqlVar])) $sql = str_replace("'$" . $sqlVar . "'", $this->dbh->quote($vars[$sqlVar]), $sql);
            }
        }
        $formVars = $this->checkSqlVar($sql, '\#');
        if($formVars)
        {
            foreach($formVars as $formVar)
            {
                if(isset($vars[$formVar])) $sql = str_replace("'#" . $formVar . "'", $this->dbh->quote($vars[$formVar]), $sql);
            }
        }
        $recordVars = $this->checkSqlVar($sql, '\@');
        if($recordVars)
        {
            foreach($recordVars as $recordVar)
            {
                if(isset($vars[$recordVar])) $sql = str_replace("'@" . $formVar . "'", $this->dbh->quote($vars[$recordVar]), $sql);
            }
        }

        $sql = $this->replaceTableNames($sql);

        try
        {
            $dataList = $this->dbh->query($sql)->fetchAll();
        }
        catch(PDOException $exception)
        {
            return $this->lang->workflowfield->error->wrongSQL . str_replace("'", "\'", $exception->getMessage());
        }
        return true;
    }

    /**
     * Get sql and vars.
     *
     * @param  string  $module
     * @param  string  $field
     * @param  string  $action
     * @access public
     * @return object
     */
    public function getSqlAndVars($module, $field = '', $action = '')
    {
        $data = $this->dao->select('*')->from(TABLE_WORKFLOWSQL)
            ->where('module')->eq($module)
            ->beginIF($field)->andWhere('field')->eq($field)->fi()
            ->beginIF($action)->andWhere('action')->eq($action)->fi()
            ->limit(1)
            ->fetch();
        if($data) $data->vars = json_decode($data->vars);
        return $data;
    }

    /**
     * Create sql and vars.
     *
     * @param  string  $module
     * @param  string  $field
     * @param  string  $action
     * @access public
     * @return void
     */
    public function createSqlAndVars($module = 0, $field = 0, $action = 0)
    {
        $sqlVars   = $this->post->sqlVars;
        $varValues = $this->post->varValues;
        if(!$sqlVars) $sqlVars = array();
        foreach($sqlVars as $varName => $sqlVar)
        {
            $sqlVar = json_decode($sqlVar);
            if(!empty($varValues[$varName])) $sqlVar->default = $varValues[$varName];
            $sqlVars[$varName] = $sqlVar;
        }

        $data = new stdclass();
        $data->module      = $module;
        $data->field       = $field;
        $data->action      = $action;
        $data->sql         = $this->post->sql;
        $data->vars        = helper::jsonEncode($sqlVars);
        $data->createdBy   = $this->app->user->account;
        $data->createdDate = helper::now();
        $this->dao->insert(TABLE_WORKFLOWSQL)->data($data)->autoCheck()->exec();

        unset($_SESSION['sqlVars']);
    }

    /**
     * Delete sql and vars.
     *
     * @param  string $module
     * @param  string $field
     * @param  string $action
     * @access public
     * @return bool
     */
    public function deleteSqlAndVars($module = 0, $field = 0, $action = 0)
    {
        $this->dao->delete()->from(TABLE_WORKFLOWSQL)
            ->where('module')->eq($module)
            ->beginIF($field)->andWhere('field')->eq($field)->fi()
            ->beginIF($action)->andWhere('action')->eq($action)->fi()
            ->exec();
        return !dao::isError();
    }

    /**
     * Get a field by ID.
     *
     * @param  int    $id
     * @param  bool   $mergeOptions
     * @access public
     * @return object
     */
    public function getByID($id, $mergeOptions = true)
    {
        $field = $this->dao->select('*')->from(TABLE_WORKFLOWFIELD)->where('id')->eq($id)->fetch();

        if($field) $field = $this->processFieldOptions($field, $mergeOptions);

        return $field;
    }

    /**
     * Get a field by field.
     *
     * @param  string $module
     * @param  string $field
     * @param  bool   $mergeOptions
     * @access public
     * @return object
     */
    public function getByField($module, $field, $mergeOptions = true)
    {
        $field = $this->dao->select('*')->from(TABLE_WORKFLOWFIELD)
            ->where('module')->eq($module)
            ->andWhere('field')->eq($field)
            ->fetch();

        if($field) $field = $this->processFieldOptions($field, $mergeOptions);

        return $field;
    }

    /**
     * Get last field.
     * 
     * @param  string $module 
     * @access public
     * @return object
     */
    public function getLastField($module)
    {
        return $this->dao->select('*')->from(TABLE_WORKFLOWFIELD)->where('module')->eq($module)->orderBy('order desc')->limit(1)->fetch();
    }

    /**
     * Get field list.
     *
     * @param  string $module
     * @param  string $orderBy
     * @access public
     * @return array
     */
    public function getList($module, $orderBy = 'order')
    {
        $fields = $this->dao->select('*')->from(TABLE_WORKFLOWFIELD)
            ->where('module')->eq($module)
            ->orderBy($orderBy)
            ->fetchAll('id');

        foreach($fields as $field) $field = $this->processFieldOptions($field);

        return $fields;
    }

    /**
     * Get field list by group.
     *
     * @param  string $orderBy
     * @access public
     * @return array
     */
    public function getGroupList($orderBy = 'order')
    {
        return $this->dao->select('*')->from(TABLE_WORKFLOWFIELD)->orderBy($orderBy)->fetchGroup('module');
    }

    /**
     * Get id => $name pairs of field.
     *
     * @param  string $module
     * @param  string $orderBy
     * @access public
     * @return array
     */
    public function getPairs($module, $orderBy = 'order')
    {
        return $this->dao->select('id, name')->from(TABLE_WORKFLOWFIELD)
            ->where('module')->eq($module)
            ->orderBy($orderBy)
            ->fetchPairs();
    }

    /**
     * Get field => name pairs.
     *
     * @param  string $module
     * @param  string $orderBy
     * @access public
     * @return array
     */
    public function getFieldPairs($module, $orderBy = 'order')
    {
        $fields = $this->dao->select('field, name')->from(TABLE_WORKFLOWFIELD)
            ->where(1)
            ->beginIF($module)->andWhere('module')->eq($module)->fi()
            ->orderBy($orderBy)
            ->fetchPairs();

        return array('' => '') + $fields;
    }

    /**
     * Get id => field pairs.
     *
     * @param  string $module
     * @access public
     * @return void
     */
    public function getCustomFields($module)
    {
        return $this->dao->select('id, field')->from(TABLE_WORKFLOWFIELD)
            ->where('module')->eq($module)
            ->andWhere('field')->notin(array_keys($this->config->workflowfield->default->fields))
            ->fetchPairs();
    }

    /**
     * Get fields can export.
     *
     * @param  string $module
     * @access public
     * @return array
     */
    public function getExportFields($module)
    {
        return $this->dao->select('id, field')->from(TABLE_WORKFLOWFIELD)->where('module')->eq($module)->andWhere('canExport')->eq('1')->fetchPairs();
    }

    /**
     * Get key field of a module.
     *
     * @param  string $module
     * @access public
     * @return string
     */
    public function getKeyField($module)
    {
        $keyField = $this->dao->select('field')->from(TABLE_WORKFLOWFIELD)->where('module')->eq($module)->andWhere('isKey')->eq('1')->fetch('field');
        if(!$keyField) $keyField = 'id';
        return $keyField;
    }

    /**
     * Get value fields of a module.
     *
     * @param  string $module
     * @access public
     * @return array
     */
    public function getValueFields($module)
    {
        $valueFields = $this->dao->select('field')->from(TABLE_WORKFLOWFIELD)->where('module')->eq($module)->andWhere('isValue')->eq('1')->orderBy('order')->fetchPairs();
        if(!$valueFields) $valueFields = array('id' => 'id');
        return $valueFields;
    }

    /**
     * Get field options.
     *
     * @param  object $field
     * @param  bool   $emptyOptions
     * @access public
     * @return array
     */
    public function getFieldOptions($field, $emptyOptions = true)
    {
        $options = (array)$field->options;
        if(zget($field, 'control') == 'select' && $emptyOptions) $options = array('' => '') + $options;
        if($field->options == 'sql')
        {
            return $this->getOptionsBySql($field->sql, $field->sqlVars);
        }
        elseif($field->options == 'user')
        {
            return $this->loadModel('user')->getDeptPairs('nodeleted,noclosed');
        }
        elseif($field->options == 'dept')
        {
            return $this->loadModel('dept')->getPairs($category = 0, $type = 'dept', 'normal');
        }
        elseif($field->options == 'prevModule')
        {
            $prevModule = $this->dao->select('prev')->from(TABLE_WORKFLOWRELATION)
                ->where('next')->eq($field->module)
                ->andWhere('field')->eq($field->field)
                ->fetch('prev');
            if(!$prevModule) return array('');;

            $prevFlow = $this->loadModel('workflow', 'flow')->getByModule($prevModule);
            if(!$prevFlow) return array('');

            return $this->loadModel('flow')->getDataPairs($prevFlow);
        }
        elseif(is_int($field->options))
        {
            $datasource = $this->loadModel('workflowdatasource', 'flow')->getByID($field->options);
            if(!$datasource) return array();

            if($datasource->type == 'option')
            {
                $options = json_decode($datasource->datasource, true);
            }
            elseif($datasource->type == 'system')
            {
                $app    = $datasource->app;
                $module = $datasource->module;
                $method = $datasource->method;

                $defaultParams = $this->workflowdatasource->getDefaultParams($app, $module, $method);
                foreach($datasource->params as $param)
                {
                    if(isset($defaultParams[$param->name])) $defaultParams[$param->name] = $param->value;
                }

                $className = $module . 'model';
                $class     = new $className();
                $options   = call_user_func_array(array($class, $method), $defaultParams);

                if(!is_array($options)) return array();

                foreach($options as $option)
                {
                    if(is_object($option) || is_array($option))
                    {
                        return array();
                    }
                }
            }
            elseif($datasource->type == 'sql')
            {
                return $this->getOptionsBySql($datasource->sql);
            }
            elseif($datasource->type == 'lang')
            {
                if($datasource->datasource == 'currency')
                {
                    return $this->loadModel('common')->getCurrencyList();
                }
                else
                {
                    $options = $this->config->workflowdatasource->langList[$datasource->datasource];
                    $app     = zget($options, 'app', '');
                    $module  = zget($options, 'module', '');
                    $field   = zget($options, 'field', '');

                    if(!$module or !$field) return array();

                    $this->app->loadLang($module, $app);
                    return isset($this->lang->$module->$field) ? $this->lang->$module->$field : array();
                }
            }
            elseif($datasource->type == 'func')
            {
            }
        }
        return $options;
    }

    /**
     * Get options by sql and sqlvars.
     *
     * @param  string $sql
     * @param  array  $sqlVars
     * @access public
     * @return array
     */
    public function getOptionsBySql($sql = '', $sqlVars = array())
    {
        foreach($sqlVars as $sqlVar)
        {
            $sql = str_replace("'$" . $sqlVar->varName . "'", $this->dbh->quote($sqlVar->default), $sql);
        }
        $sql = $this->replaceTableNames($sql);

        try
        {
            $options  = array('' => '');
            $dataList = $this->dbh->query($sql)->fetchAll();
            foreach($dataList as $data)
            {
                $data = (array)$data;
                if(count($data) > 1)
                {
                    $key   = current($data);
                    $value = next($data);
                    $options[$key] = $value;
                }
                elseif(count($data) > 0)
                {
                    $key   = current($data);
                    $value = current($data);
                    $options[$key] = $value;
                }
            }

            return $options;
        }
        catch(PDOException $exception)
        {
            return array();
        }
    }

    /**
     * Process field length.
     *
     * @param  object $field
     * @access public
     * @return array | object
     */
    public function processFieldLength($field = null)
    {
        $error = array();
        $field->length = str_replace(array('.','，'), ',', $field->length);
        if($field->type == 'decimal')
        {
            if(strpos($field->length, ',') === false)
            {
                $field->length = (int)$field->length;
                if($field->length < 0 or $field->length > 65) $error['length'] = $this->lang->workflowfield->error->wrongDecimal;
                $field->length .= ',0';
            }
            else
            {
                $lengthList = explode(',', $field->length);
                if(count($lengthList) > 2)
                {
                    $error['length'] = $this->lang->workflowfield->error->wrongDecimal;
                }
                else
                {
                    $a = (int)$lengthList[0];
                    $b = (int)$lengthList[1];
                    if($a < $b)
                    {
                        $error['length'] = $this->lang->workflowfield->error->wrongDecimal;
                    }
                    else
                    {
                        if($a < 0 or $a > 65 or $b < 0 or $b > 30) $error['length'] = $this->lang->workflowfield->error->wrongDecimal;
                    }
                    $field->length = $a . ',' . $b;
                }
            }
            if((int)$field->length == 0) $field->length = '10,0';
        }
        elseif($field->type == 'char')
        {
            $field->length = (int)$field->length;
            if($field->length < 0 or $field->length > 255) $error['length'] = $this->lang->workflowfield->error->wrongChar;
        }
        elseif($field->type == 'varchar')
        {
            $field->length = (int)$field->length;
            if($field->length < 0 or $field->length > 21844) $error['length'] = $this->lang->workflowfield->error->wrongVarchar;
        }
        if($error) return array('result' => 'fail', 'message' => $error);
        return $field;
    }

    /**
     * Process field options.
     *
     * @param  object $field
     * @param  bool   $mergeOptions
     * @access public
     * @return object
     */
    public function processFieldOptions($field, $mergeOptions = true)
    {
        /* Init sql and sql vars. */
        $field->sql     = '';
        $field->sqlVars = array();

        if($field->options == 'sql')
        {
            $data = $this->getSqlAndVars($field->module, $field->field);
            if($data)
            {
                $field->sql     = $data->sql;
                $field->sqlVars = $data->vars;
            }

            return $field;
        }

        if($field->options == 'user')       return $field;
        if($field->options == 'dept')       return $field;
        if($field->options == 'prevModule') return $field;
        if(is_int($field->options))         return $field;

        if(is_string($field->options)) $field->options = json_decode($field->options, true);
        if(!$field->options) $field->options = (array)$field->options;

        /* 把子状态的options由二维数组转成一维数组。 */
        if($mergeOptions && $field->field == 'subStatus' && is_array($field->options))
        {
            foreach($field->options as $parent => $subStatus)
            {
                unset($field->options[$parent]);

                $options = zget($subStatus, 'options', array());

                foreach($options as $code => $name) $field->options[$code] = $name;
            }
        }

        return $field;
    }

    /**
     * Create a field.
     *
     * @param  string $module
     * @access public
     * @return bool|array
     */
    public function create($module = '')
    {
        $flow = $this->loadModel('workflow')->getByModule($module);
        if(!$flow) return false;

        $orderField = $this->getByID($this->post->order);

        $fixer = fixer::input('post')
            ->add('module', $module)
            ->add('createdBy', $this->app->user->account)
            ->add('createdDate', helper::now())
            ->setForce('order', $orderField->order + 1);
        $fixer = $this->processField($fixer);
        $field = $fixer->remove('sql, varValues, sqlVars')->get();

        $result = $this->checkField($field);
        if(is_array($result) && zget($result, 'result') == 'fail') return $result;
        $field = $result;

        $this->dao->insert(TABLE_WORKFLOWFIELD)->data($field, $skip = 'optionType')->autoCheck()
            ->check('field', 'unique', "module='$module'")
            ->batchCheck($this->config->workflowfield->require->create, 'notempty')
            ->exec();
        $id = $this->dao->lastInsertId();

        if(dao::isError()) return false;

        if($field->length) $field->type .= "($field->length)";
        $sql = "ALTER TABLE `$flow->table` ADD `$field->field` $field->type NOT NULL";
        if($field->type != 'text' && $field->default != '')
        {
            $magicQuote = (version_compare(phpversion(), '5.4', '<') and function_exists('get_magic_quotes_gpc') and get_magic_quotes_gpc());
            if($magicQuote) $field->default = stripslashes($field->default);
            $field->default = $this->dbh->quote($field->default);
            $sql .= " DEFAULT $field->default";
        }
        $sql .= " AFTER `$orderField->field`";

        try
        {
            $this->dbh->query($sql);
        }
        catch(PDOException $exception)
        {
            $this->dao->delete()->from(TABLE_WORKFLOWFIELD)->where('id')->eq($id)->exec();
            return array('result' => 'fail', 'message' => $exception->getMessage());
        }

        /* Update other fields's order. */
        $this->dao->update(TABLE_WORKFLOWFIELD)->set('`order` = `order` + 1')
            ->where('id')->ne($id)
            ->andWhere('`order`')->ge($field->order)
            ->exec();

        /* Create sql and vars. */
        if($field->optionType == 'sql') $this->createSqlAndVars($module, $field->field);

        return $id;
    }

    /**
     * Update a field.
     *
     * @param  int    $id
     * @access public
     * @return bool|array
     */
    public function update($id)
    {
        $oldField = $this->getByID($id);
        if(!$oldField) return false;

        $flow = $this->loadModel('workflow', 'flow')->getByModule($oldField->module);
        if(!$flow) return false;

        /* 内置字段中subStatus可以编辑，其他不可编辑。默认字段中status、subStatus可以编辑，其他不可编辑。非内置且非默认字段可以编辑。*/
        /* The subStatus in the built-in field can be edited, and the others cannot be edited. The status, status, and status in the default field can be edited, and others cannot be edited. Non-built-in and non-default fields can be edited. */
        $cannotEdit = (($oldField->buildin && $oldField->field != 'subStatus') or (!$oldField->buildin && in_array($oldField->field, $this->config->workflowfield->cannotEditFields)));
        if($cannotEdit)
        {
            $this->dao->update(TABLE_WORKFLOWFIELD)->set('name')->eq($this->post->name)->autoCheck()->where('id')->eq($id)->exec();
            return !dao::isError();
        }

        $orderField = $this->getByID($this->post->order);

        $fixer = fixer::input('post')
            ->add('editedBy', $this->app->user->account)
            ->add('editedDate', helper::now())
            ->setIF($this->post->order != $this->post->oldOrder, 'order', $orderField->order + 1);
        $fixer = $this->processField($fixer);
        $field = $fixer->setIF(!$this->post->rules, 'rules', '')
            ->removeIF($this->post->order == $this->post->oldOrder, 'order')
            ->remove('oldOrder, sql, varValues, sqlVars, keyValue, parentCode, parentName, optionCode, optionName, optionDefault')
            ->get();

        $result = $this->checkField($field);
        if(is_array($result) && zget($result, 'result') == 'fail') return $result;
        $field = $result;

        $this->dao->update(TABLE_WORKFLOWFIELD)->data($field, $skip = 'optionType')->autoCheck()
            ->batchCheck($this->config->workflowfield->require->edit, 'notempty')
            ->where('id')->eq($id)
            ->exec();

        if(dao::isError()) return false;

        /* Update other fields's order. */
        if(zget($field, 'order', $oldField->order) != $oldField->order)
        {
            $this->dao->update(TABLE_WORKFLOWFIELD)->set('`order` = `order` + 1')
                ->where('id')->ne($id)
                ->andWhere('`order`')->ge($field->order)
                ->exec();
        }

        $result = $this->processTable($flow, $oldField, $field, $orderField);
        if(is_array($result)) return $result;

        if($oldField->field != $field->field)
        {
            $this->dao->update(TABLE_WORKFLOWLAYOUT)->set('field')->eq($field->field)
                ->where('module')->eq($oldField->module)
                ->andWhere('field')->eq($oldField->field)
                ->exec();
        }

        if($field->optionType == 'sql')
        {
            $this->deleteSqlAndVars($oldField->module, $oldField->field);
            /* Create sql and vars. */
            $this->createSqlAndVars($oldField->module, $field->field);
        }

        $this->loadModel('workflowrelation', 'flow')->updateField($flow->module, $orderField->field, $field->field);

        return true;
    }

    /**
     * Process a field.
     *
     * @param  object $fixer
     * @access public
     * @return object
     */
    public function processField($fixer)
    {
        $field = $fixer->join('rules', ',')
            ->setForce('field', str_replace(' ', '', $this->post->field))
            ->setForce('options', '[]');

        if(empty($field->isValue)) $field->isValue = 0;
        if(empty($field->length))
        {
            $field->length = 0;
            if($field->type == 'char')    $field->length = 50;
            if($field->type == 'varchar') $field->length = 255;
        }

        if($field->control == 'input' && $field->type != 'char' && $field->type != 'varchar')
        {
            $field->type   = 'varchar';
            $field->length = 255;
        }
        if($field->control == 'date' && $field->type != 'date')
        {
            $field->type   = 'date';
            $field->length = 0;
        }
        if($field->control == 'datetime' && $field->type != 'datetime')
        {
            $field->type   = 'datetime';
            $field->length = 0;
        }
        if($field->control == 'textarea' && $field->type != 'text')
        {
            $field->type   = 'text';
            $field->length = 0;
        }
        if($field->control == 'checkbox' && $field->type != 'varchar')
        {
            $field->type   = 'varchar';
            $field->length = 255;
        }

        return $field;
    }

    /**
     * Process table of the flow.
     *
     * @param  object $flow
     * @param  object $oldField
     * @param  object $field
     * @param  object $orderField
     * @access public
     * @return bool | array
     */
    public function processTable($flow, $oldField, $field, $orderField = null)
    {
        $sql = '';
        if($oldField->field != $field->field || $oldField->type != $field->type || $oldField->length != $field->length || $oldField->control != $field->control || $oldField->default != $field->default || zget($field, 'order', $oldField->order) != $oldField->order)
        {
            if($field->length and strpos("|decimal|char|varchar|", "|{$field->type}|") !== false) $field->type .= "($field->length)";

            $sql = "ALTER TABLE `$flow->table` CHANGE `$oldField->field` `$field->field` $field->type NOT NULL";

            if($field->type != 'text' && $field->default)
            {
                $magicQuote = (version_compare(phpversion(), '5.4', '<') and function_exists('get_magic_quotes_gpc') and get_magic_quotes_gpc());
                if($magicQuote) $field->default = stripslashes($field->default);

                $field->default = $this->dbh->quote($field->default);

                $sql .= " DEFAULT $field->default";
            }

            if(!empty($orderField->field)) $sql .= " AFTER `$orderField->field`";
        }

        if(!$sql) return true;

        try
        {
            $this->dbh->query($sql);

            return true;
        }
        catch(PDOException $exception)
        {
            if(is_array($oldField->options) or is_object($oldField->options))
            {
                $oldField->options = helper::jsonEncode($oldField->options);
            }

            $this->dao->update(TABLE_WORKFLOWFIELD)->data($oldField, $skip = 'id, module, placeholder, buildin, desc, createdBy, createdDate, editedBy, editedDate, sql, sqlVars')->autoCheck()
                ->batchCheck($this->config->workflowfield->require->edit, 'notempty')
                ->where('id')->eq($oldField->id)
                ->exec();

            return array('result' => 'fail', 'message' => $exception->getMessage());
        }
    }

    /**
     * Check a field.
     *
     * @param  object $field
     * @access public
     * @return array | object 
     */
    public function checkField($field)
    {
        if(!empty($field->field) && !validater::checkREG($field->field, '|^[A-Za-z]+$|')) return array('result' => 'fail', 'message' => array('field' => sprintf($this->lang->workflowfield->error->wrongCode, $this->lang->workflowfield->field)));

        $result = $this->processFieldLength($field);
        if(is_array($result) && $result['result'] == 'fail') return $result;

        $field = $result;

        if(strpos(',select,radio,checkbox,', ",$field->control,") !== false)
        {
            if($field->optionType == 'sql')
            {
                $result = $this->checkSqlAndVars($this->post->sql, $this->post->varValues);
                if($result !== true) return array('result' => 'fail', 'message' => array('sql' => $result));
            }
            if($field->optionType == 'custom')
            {
                $options = array();
                if($field->field == 'subStatus')
                {
                    $errors     = array();  // Log errors.
                    $duplicated = array();  // Log duplicated codes.

                    if(!$this->post->parentCode) return array('result' => 'fail', 'message' => $this->lang->workflowfield->tips->emptyStatus);

                    foreach($this->post->parentCode as $key => $parentCode)
                    {
                        $parentName = $this->post->parentName[$key];

                        if($parentCode == '' or $parentName == '') continue;

                        /* Init the default property and options property for a parent status. */
                        $options[$parentCode]['default'] = zget($this->post->optionDefault, $parentCode, '');
                        $options[$parentCode]['options'] = array();

                        foreach($this->post->optionCode[$parentCode] as $key => $code)
                        {
                            if(strlen($code) > 30)
                            {
                                return array('result' => 'fail', 'message' => array("optionDefault$parentCode" => sprintf($this->lang->workflowfield->error->longCode, $this->lang->workflowfield->key)));
                            }

                            $name = $this->post->optionName[$parentCode][$key];

                            if($code == '' or $name == '') continue;

                            /* Check if the code of sub status is duplicated. */
                            $searchKey = array_search($code, $this->post->optionCode[$parentCode]);
                            if($searchKey !== false && $searchKey != $key) $duplicated[] = $code;

                            $options[$parentCode]['options'][$code] = $name;
                        }

                        $optionsCount = count($options[$parentCode]['options']);

                        if(!$options[$parentCode]['default'])
                        {
                            /* If the sub status is empty, set parent status as sub status. */
                            if($optionsCount == 0)
                            {
                                $options[$parentCode]['options'][$parentCode] = $parentName;
                                $options[$parentCode]['default'] = $parentCode;
                            }

                            if($optionsCount == 1) $options[$parentCode]['default'] = key($options[$parentCode]['options']);

                            if($optionsCount > 1) $errors["optionDefault$parentCode"] = $this->lang->workflowfield->error->emptyDefault; 
                        }
                    }

                    if($duplicated) $errors['optionsDIV'] = sprintf($this->lang->workflowfield->error->duplicatedCode, implode(',', array_unique($duplicated)));

                    if($errors) return array('result' => 'fail', 'message' => $errors);
                }
                elseif($this->post->options)
                {
                    $longCode = false;
                    foreach($this->post->options['code'] as $key => $code)
                    {
                        if(strlen($code) > 30)
                        {
                            return array('result' => 'fail', 'message' => array('optionsDIV' => sprintf($this->lang->workflowfield->error->longCode, $this->lang->workflowfield->key)));
                        }

                        $name = $this->post->options['name'][$key];

                        if($code == '' or $name == '') continue;

                        $options[$code] = $name;
                    }
                }

                /* If options is empty, return error. */
                if(empty($options))
                {
                    return array('result' => 'fail', 'message' => array('optionsDIV' => $this->lang->workflowfield->error->emptyOptions));
                }
                $field->options = helper::jsonEncode($options);
            }
            else
            {
                $field->options = $field->optionType;
            }
        }

        if($field->default)
        {
            if($field->type == 'text') return array('result' => 'fail', 'message' => $this->lang->workflowfield->error->textDefaultValue);
            foreach($this->config->workflowfield->lengthList as $length => $controlList)
            {
                if(strpos($controlList, ",{$field->control},") !== false and strlen($field->default) > $length)
                {
                    return array('result' => 'fail', 'message' => sprintf($this->lang->workflowfield->error->defaultValue, $length));
                }
            }
        }

        /* If this options's value of field is user, set value of type and value of length. */
        if($field->options == 'user')
        {
            $field->type   = 'varchar';
            $field->length = 30;
        }

        return $field;
    }

    /**
     * Delete a field.
     *
     * @param  int    $id
     * @param  object $null
     * @access public
     * @return bool
     */
    public function delete($id, $null = null)
    {
        $field = $this->getByID($id);
        if(!$field) return false;

        $flow = $this->loadModel('workflow', 'flow')->getByModule($field->module);
        if(!$flow) return false;

        $this->dao->delete()->from(TABLE_WORKFLOWFIELD)->where('id')->eq($id)->exec();
        if(dao::isError()) return false;

        $this->dao->update(TABLE_WORKFLOWFIELD)->set('`order` = `order` - 1')->where('`order`')->gt($field->order)->exec();

        $this->dao->delete()->from(TABLE_WORKFLOWLAYOUT)
            ->where('module')->eq($field->module)
            ->andWhere('field')->eq($field->field)
            ->exec();
        if(dao::isError()) return false;

        $sql = "ALTER TABLE `$flow->table` DROP `$field->field`";
        if(!$this->dbh->query($sql)) return false;

        if($field->options == 'sql') $this->deleteSqlAndVars($field->module, $field->field);
        return true;
    }

    /**
     * Set fields to export.
     *
     * @param  string $module
     * @access public
     * @return void
     */
    public function setExport($module)
    {
        $this->dao->update(TABLE_WORKFLOWFIELD)->set('canExport')->eq('0')
            ->where('module')->eq($module)
            ->andWhere('field')->notin($this->post->fields)
            ->exec();

        $this->dao->update(TABLE_WORKFLOWFIELD)->set('canExport')->eq('1')
            ->where('module')->eq($module)
            ->andWhere('field')->in($this->post->fields)
            ->exec();

        return !dao::isError();
    }
}
